@@ -17,7 +17,7 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2021 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the license.  You may  %
@@ -36,7 +36,7 @@
 */
 
 /*
-  include declarations.
+  Include declarations.
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
@@ -48,6 +48,7 @@
 #include "magick/locale_.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
+#include "magick/memory-private.h"
 #include "magick/nt-base.h"
 #include "magick/nt-base-private.h"
 #include "magick/property.h"
@@ -139,7 +140,6 @@ MagickExport char *AcquireString(const char *source)
     sizeof(*destination));
   if (destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  *destination='\0';
   if (source != (char *) NULL)
     (void) memcpy(destination,source,length*sizeof(*destination));
   destination[length]='\0';
@@ -341,6 +341,8 @@ MagickExport StringInfo *CloneStringInfo(const StringInfo *string_info)
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickCoreSignature);
   clone_info=AcquireStringInfo(string_info->length);
+  (void) strcpy(clone_info->path,string_info->path);
+  (void) CloneString(&clone_info->name,string_info->name);
   if (string_info->length != 0)
     (void) memcpy(clone_info->datum,string_info->datum,string_info->length+1);
   return(clone_info);
@@ -409,8 +411,8 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 %
 %  The format of the ConcatenateMagickString method is:
 %
-%      size_t ConcatenateMagickString(char *destination,const char *source,
-%        const size_t length)
+%      size_t ConcatenateMagickString(char *magick_restrict destination,
+%        const char *magick_restrict source,const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -421,16 +423,16 @@ MagickExport int CompareStringInfo(const StringInfo *target,
 %    o length: the length of the destination string.
 %
 */
-MagickExport size_t ConcatenateMagickString(char *destination,
-  const char *source,const size_t length)
+MagickExport size_t ConcatenateMagickString(char *magick_restrict destination,
+  const char *magick_restrict source,const size_t length)
 {
-  register char
-    *q;
+  char
+    *magick_restrict q;
 
-  register const char
-    *p;
+  const char
+    *magick_restrict p;
 
-  register size_t
+  size_t
     i;
 
   size_t
@@ -477,8 +479,8 @@ MagickExport size_t ConcatenateMagickString(char *destination,
 %
 %  The format of the ConcatenateString method is:
 %
-%      MagickBooleanType ConcatenateString(char **destination,
-%        const char *source)
+%      MagickBooleanType ConcatenateString(char **magick_restrict destination,
+%        const char *magick_restrict source)
 %
 %  A description of each parameter follows:
 %
@@ -487,8 +489,8 @@ MagickExport size_t ConcatenateMagickString(char *destination,
 %    o source: A character string.
 %
 */
-MagickExport MagickBooleanType ConcatenateString(char **destination,
-  const char *source)
+MagickExport MagickBooleanType ConcatenateString(
+  char **magick_restrict destination,const char *magick_restrict source)
 {
   size_t
     destination_length,
@@ -511,8 +513,8 @@ MagickExport MagickBooleanType ConcatenateString(char **destination,
   length+=source_length;
   if (~length < MaxTextExtent)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
-  *destination=(char *) ResizeQuantumMemory(*destination,length+MaxTextExtent,
-    sizeof(**destination));
+  *destination=(char *) ResizeQuantumMemory(*destination,
+    OverAllocateMemory(length+MaxTextExtent),sizeof(**destination));
   if (*destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
   if (source_length != 0)
@@ -559,8 +561,20 @@ MagickExport void ConcatenateStringInfo(StringInfo *string_info,
   length=string_info->length;
   if (~length < source->length)
     ThrowFatalException(ResourceLimitFatalError,"UnableToConcatenateString");
-  SetStringInfoLength(string_info,length+source->length);
-  (void) memcpy(string_info->datum+length,source->datum,source->length);
+  length+=source->length;
+  if (~length < MagickPathExtent)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  if (string_info->datum == (unsigned char *) NULL)
+    string_info->datum=(unsigned char *) AcquireQuantumMemory(length+
+      MagickPathExtent,sizeof(*string_info->datum));
+  else
+    string_info->datum=(unsigned char *) ResizeQuantumMemory( 
+      string_info->datum,OverAllocateMemory(length+MagickPathExtent),
+      sizeof(*string_info->datum));
+  if (string_info->datum == (unsigned char *) NULL)
+    ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) memcpy(string_info->datum+string_info->length,source->datum,source->length);
+  string_info->length=length;
 }
 
 /*
@@ -634,7 +648,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
     }
   else
     {
-      register size_t
+      size_t
         i;
 
       ssize_t
@@ -644,7 +658,7 @@ MagickExport StringInfo *ConfigureFileToStringInfo(const char *filename)
       for (i=0; i < length; i+=count)
       {
         count=read(file,string+i,(size_t) MagickMin(length-i,(size_t)
-          SSIZE_MAX));
+          MAGICK_SSIZE_MAX));
         if (count <= 0)
           {
             count=0;
@@ -710,7 +724,6 @@ MagickExport char *ConstantString(const char *source)
     destination=(char *) AcquireQuantumMemory(length+1UL,sizeof(*destination));
   if (destination == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
-  *destination='\0';
   if (source != (char *) NULL)
     (void) memcpy(destination,source,length*sizeof(*destination));
   destination[length]='\0';
@@ -732,13 +745,12 @@ MagickExport char *ConstantString(const char *source)
 %  out exceeding the given pre-declared length.
 %
 %  The destination buffer is always null-terminated even if the string must be
-%  truncated.  The return value is the minimum of the source string length or
-%  the length parameter.
+%  truncated.  The return value is length of the string.
 %
 %  The format of the CopyMagickString method is:
 %
-%      size_t CopyMagickString(const char *destination,char *source,
-%        const size_t length)
+%      size_t CopyMagickString(const char *magick_restrict destination,
+%        char *magick_restrict source,const size_t length)
 %
 %  A description of each parameter follows:
 %
@@ -749,50 +761,39 @@ MagickExport char *ConstantString(const char *source)
 %    o length: the length of the destination string.
 %
 */
-MagickExport size_t CopyMagickString(char *destination,const char *source,
-  const size_t length)
+MagickExport size_t CopyMagickString(char *magick_restrict destination,
+  const char *magick_restrict source,const size_t length)
 {
-  register char
-    *q;
+  char
+    *magick_restrict q;
 
-  register const char
-    *p;
+  const char
+    *magick_restrict p;
 
-  register size_t
+  size_t
     n;
 
   p=source;
   q=destination;
   for (n=length; n > 4; n-=4)
   {
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
-    *q=(*p++);
-    if (*q == '\0')
+    if (((*q++)=(*p++)) == '\0')
       return((size_t) (p-source-1));
-    q++;
   }
-  if (n != 0)
-    for (n--; n != 0; n--)
-    {
-      *q=(*p++);
-      if (*q == '\0')
-        return((size_t) (p-source-1));
-      q++;
-    }
   if (length != 0)
-    *q='\0';
-  return((size_t) (p-source-1));
+    {
+      while (--n != 0)
+        if (((*q++)=(*p++)) == '\0')
+          return((size_t) (p-source-1));
+      *q='\0';
+    }
+  return((size_t) (p-source));
 }
 
 /*
@@ -882,7 +883,7 @@ MagickExport StringInfo *DestroyStringInfo(StringInfo *string_info)
 */
 MagickExport char **DestroyStringList(char **list)
 {
-  register ssize_t
+  ssize_t
     i;
 
   assert(list != (char **) NULL);
@@ -926,10 +927,10 @@ MagickExport char *EscapeString(const char *source,const char escape)
   char
     *destination;
 
-  register char
+  char
     *q;
 
-  register const char
+  const char
     *p;
 
   size_t
@@ -1093,7 +1094,7 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
     bytes,
     length;
 
-  register ssize_t
+  ssize_t
     i;
 
   ssize_t
@@ -1141,86 +1142,6 @@ MagickExport ssize_t FormatMagickSize(const MagickSizeType size,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%  F o r m a t M a g i c k T i m e                                            %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  FormatMagickTime() returns the specified time in the Internet date/time
-%  format and the length of the timestamp.
-%
-%  The format of the FormatMagickTime method is:
-%
-%      ssize_t FormatMagickTime(const time_t time,const size_t length,
-%        char *timestamp)
-%
-%  A description of each parameter follows.
-%
-%   o time:  the time since the Epoch (00:00:00 UTC, January 1, 1970),
-%     measured in seconds.
-%
-%   o length: the maximum length of the string.
-%
-%   o timestamp:  Return the Internet date/time here.
-%
-*/
-MagickExport ssize_t FormatMagickTime(const time_t time,const size_t length,
-  char *timestamp)
-{
-  ssize_t
-    count;
-
-  struct tm
-    gm_time,
-    local_time;
-
-  time_t
-    timezone;
-
-  assert(timestamp != (char *) NULL);
-  (void) memset(&local_time,0,sizeof(local_time));
-  (void) memset(&gm_time,0,sizeof(gm_time));
-#if defined(MAGICKCORE_HAVE_LOCALTIME_R)
-  (void) localtime_r(&time,&local_time);
-#else
-  {
-    struct tm
-      *my_time;
-
-    my_time=localtime(&time);
-    if (my_time != (struct tm *) NULL)
-      (void) memcpy(&local_time,my_time,sizeof(local_time));
-  }
-#endif
-#if defined(MAGICKCORE_HAVE_GMTIME_R)
-  (void) gmtime_r(&time,&gm_time);
-#else
-  {
-    struct tm
-      *my_time;
-
-    my_time=gmtime(&time);
-    if (my_time != (struct tm *) NULL)
-      (void) memcpy(&gm_time,my_time,sizeof(gm_time));
-  }
-#endif
-  timezone=(time_t) ((local_time.tm_min-gm_time.tm_min)/60+
-    local_time.tm_hour-gm_time.tm_hour+24*((local_time.tm_year-
-    gm_time.tm_year) != 0 ? (local_time.tm_year-gm_time.tm_year) :
-    (local_time.tm_yday-gm_time.tm_yday)));
-  count=FormatLocaleString(timestamp,length,
-    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_time.tm_year+1900,
-    local_time.tm_mon+1,local_time.tm_mday,local_time.tm_hour,
-    local_time.tm_min,local_time.tm_sec,(long) timezone);
-  return(count);
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   G e t E n v i r o n m e n t V a l u e                                     %
 %                                                                             %
 %                                                                             %
@@ -1569,10 +1490,10 @@ MagickExport MagickBooleanType IsStringNotFalse(const char *value)
 MagickExport void PrintStringInfo(FILE *file,const char *id,
   const StringInfo *string_info)
 {
-  register const char
+  const char
     *p;
 
-  register size_t
+  size_t
     i,
     j;
 
@@ -1668,10 +1589,10 @@ MagickExport void ResetStringInfo(StringInfo *string_info)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  SanitizeString() returns an new string removes all characters except
+%  SanitizeString() returns a new string with all characters removed except
 %  letters, digits and !#$%&'*+-=?^_`{|}~@.[].
 %
-%  The returned string shoud be freed using DestoryString().
+%  Free the sanitized string with DestroyString().
 %
 %  The format of the SanitizeString method is:
 %
@@ -1690,18 +1611,18 @@ MagickExport char *SanitizeString(const char *source)
   const char
     *q;
 
-  register char
+  char
     *p;
 
   static char
-    whitelist[] =
+    allowlist[] =
       "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 "
       "$-_.+!*'(),{}|\\^~[]`\"><#%;/?:@&=";
 
   sanitize_source=AcquireString(source);
   p=sanitize_source;
   q=sanitize_source+strlen(sanitize_source);
-  for (p+=strspn(p,whitelist); p != q; p+=strspn(p,whitelist))
+  for (p+=strspn(p,allowlist); p != q; p+=strspn(p,allowlist))
     *p='_';
   return(sanitize_source);
 }
@@ -1995,13 +1916,13 @@ MagickExport char *StringInfoToHexString(const StringInfo *string_info)
   char
     *string;
 
-  register const unsigned char
+  const unsigned char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
-  register unsigned char
+  unsigned char
     *q;
 
   size_t
@@ -2083,11 +2004,11 @@ MagickExport char **StringToArgv(const char *text,int *argc)
   char
     **argv;
 
-  register const char
+  const char
     *p,
     *q;
 
-  register ssize_t
+  ssize_t
     i;
 
   *argc=0;
@@ -2205,7 +2126,7 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
   double
     *array;
 
-  register ssize_t
+  ssize_t
     i;
 
   /*
@@ -2271,13 +2192,13 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
 %
 %  StringToken() looks for any one of given delimiters and splits the string
 %  into two separate strings by replacing the delimiter character found with a
-%  nul character.
+%  null character.
 %
 %  The given string pointer is changed to point to the string following the
 %  delimiter character found, or NULL.  A pointer to the start of the
 %  string is returned, representing the token before the delimiter.
 %
-%  In may ways this is equivent to the strtok() C library function, but with
+%  StringToken() is equivent to the strtok() C library method, but with
 %  multiple delimiter characters rather than a delimiter string.
 %
 %  The format of the StringToken method is:
@@ -2297,13 +2218,13 @@ MagickExport char *StringToken(const char *delimiters,char **string)
   char
     *q;
 
-  register char
+  char
     *p;
 
-  register const char
+  const char
     *r;
 
-  register int
+  int
     c,
     d;
 
@@ -2356,28 +2277,63 @@ MagickExport char *StringToken(const char *delimiters,char **string)
 %
 */
 MagickExport char **StringToList(const char *text)
+{
+  return(StringToStrings(text,(size_t *) NULL));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%  S t r i n g T o S t r i n g s                                              %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  StringToStrings() converts a text string into a list by segmenting the text
+%  string at each carriage return discovered.  The list is converted to HEX
+%  characters if any control characters are discovered within the text string.
+%
+%  The format of the StringToList method is:
+%
+%      char **StringToList(const char *text,size_t *lines)
+%
+%  A description of each parameter follows:
+%
+%    o text:  Specifies the string to segment into a list.
+%
+%    o count: Return value for the number of items in the list.
+%
+*/
+MagickExport char **StringToStrings(const char *text,size_t *count)
 {
   char
     **textlist;
 
-  register const char
+  const char
     *p;
 
-  register ssize_t
+  ssize_t
     i;
 
   size_t
     lines;
 
   if (text == (char *) NULL)
-    return((char **) NULL);
+    {
+      if (count != (size_t *) NULL)
+        *count=0;
+      return((char **) NULL);
+    }
   for (p=text; *p != '\0'; p++)
     if (((int) ((unsigned char) *p) < 32) &&
         (isspace((int) ((unsigned char) *p)) == 0))
       break;
   if (*p == '\0')
     {
-      register const char
+      const char
         *q;
 
       /*
@@ -2411,12 +2367,12 @@ MagickExport char **StringToList(const char *text)
   else
     {
       char
-        hex_string[MaxTextExtent];
+        hex_string[MagickPathExtent];
 
-      register char
+      char
         *q;
 
-      register ssize_t
+      ssize_t
         j;
 
       /*
@@ -2430,17 +2386,21 @@ MagickExport char **StringToList(const char *text)
       p=text;
       for (i=0; i < (ssize_t) lines; i++)
       {
-        textlist[i]=(char *) AcquireQuantumMemory(2UL*MaxTextExtent,
+        size_t
+          length;
+
+        textlist[i]=(char *) AcquireQuantumMemory(2UL*MagickPathExtent,
           sizeof(**textlist));
         if (textlist[i] == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
-        (void) FormatLocaleString(textlist[i],MaxTextExtent,"0x%08lx: ",
+        (void) FormatLocaleString(textlist[i],MagickPathExtent,"0x%08lx: ",
           (long) (CharsPerLine*i));
         q=textlist[i]+strlen(textlist[i]);
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),CharsPerLine); j++)
+        length=strlen(p);
+        for (j=1; j <= (ssize_t) MagickMin(length,CharsPerLine); j++)
         {
-          (void) FormatLocaleString(hex_string,MaxTextExtent,"%02x",*(p+j));
-          (void) CopyMagickString(q,hex_string,MaxTextExtent);
+          (void) FormatLocaleString(hex_string,MagickPathExtent,"%02x",*(p+j));
+          (void) CopyMagickString(q,hex_string,MagickPathExtent);
           q+=2;
           if ((j % 0x04) == 0)
             *q++=' ';
@@ -2453,7 +2413,7 @@ MagickExport char **StringToList(const char *text)
             *q++=' ';
         }
         *q++=' ';
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),CharsPerLine); j++)
+        for (j=1; j <= (ssize_t) MagickMin(length,CharsPerLine); j++)
         {
           if (isprint((int) ((unsigned char) *p)) != 0)
             *q++=(*p);
@@ -2462,12 +2422,14 @@ MagickExport char **StringToList(const char *text)
           p++;
         }
         *q='\0';
-        textlist[i]=(char *) ResizeQuantumMemory(textlist[i],q-textlist[i]+1,
-          sizeof(**textlist));
+        textlist[i]=(char *) ResizeQuantumMemory(textlist[i],(size_t) (q-
+          textlist[i]+1),sizeof(**textlist));
         if (textlist[i] == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
       }
     }
+  if (count != (size_t *) NULL)
+    *count=lines;
   textlist[i]=(char *) NULL;
   return(textlist);
 }
@@ -2530,7 +2492,7 @@ MagickExport StringInfo *StringToStringInfo(const char *string)
 */
 MagickExport void StripString(char *message)
 {
-  register char
+  char
     *p,
     *q;
 
@@ -2596,7 +2558,7 @@ MagickExport MagickBooleanType SubstituteString(char **string,
   MagickBooleanType
     status;
 
-  register char
+  char
     *p;
 
   size_t
@@ -2629,8 +2591,8 @@ MagickExport MagickBooleanType SubstituteString(char **string,
         */
         offset=(ssize_t) (p-(*string));
         extent=strlen(*string)+replace_extent-search_extent+1;
-        *string=(char *) ResizeQuantumMemory(*string,extent+MaxTextExtent,
-          sizeof(*p));
+        *string=(char *) ResizeQuantumMemory(*string,
+          OverAllocateMemory(extent+MaxTextExtent),sizeof(*p));
         if (*string == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
         p=(*string)+offset;
